# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from .RevertReceiveByUserId import RevertReceiveByUserId
from .IncreaseCounterByUserId import IncreaseCounterByUserId
from .SetCounterByUserId import SetCounterByUserId
from .ReceiveByUserId import ReceiveByUserId
from .DecreaseCounterByUserId import DecreaseCounterByUserId
from .VerifyCompleteByUserId import VerifyCompleteByUserId
from .VerifyCounterValueByUserId import VerifyCounterValueByUserId


# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *
from .Version import Version
from .options.SignTargetVersionOptions import SignTargetVersionOptions


class SignTargetVersion:
    region: str
    owner_id: str
    namespace_name: str
    version_name: str
    version: Version

    def __init__(
        self,
        region: str,
        owner_id: str,
        namespace_name: str,
        version_name: str,
        version: Version,
        options: Optional[SignTargetVersionOptions] = SignTargetVersionOptions(),
    ):
        self.region = region
        self.owner_id = owner_id
        self.namespace_name = namespace_name
        self.version_name = version_name
        self.version = version

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.region is not None:
            properties["region"] = self.region
        if self.owner_id is not None:
            properties["ownerId"] = self.owner_id
        if self.namespace_name is not None:
            properties["namespaceName"] = self.namespace_name
        if self.version_name is not None:
            properties["versionName"] = self.version_name
        if self.version is not None:
            properties["version"] = self.version.properties(
            )

        return properties



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *
from .options.PrizeLimitOptions import PrizeLimitOptions


class PrizeLimit:
    prize_id: str
    drawn_count: int
    revision: Optional[int] = None

    def __init__(
        self,
        prize_id: str,
        drawn_count: int,
        options: Optional[PrizeLimitOptions] = PrizeLimitOptions(),
    ):
        self.prize_id = prize_id
        self.drawn_count = drawn_count
        self.revision = options.revision if options.revision else None

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.prize_id is not None:
            properties["prizeId"] = self.prize_id
        if self.drawn_count is not None:
            properties["drawnCount"] = self.drawn_count

        return properties



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from .Namespace import Namespace
from .options.NamespaceOptions import NamespaceOptions
from .MatchSession import MatchSession
from .options.MatchSessionOptions import MatchSessionOptions
from .SeasonModel import SeasonModel
from .options.SeasonModelOptions import SeasonModelOptions
from .TierModel import TierModel
from .options.TierModelOptions import TierModelOptions
from .GameResult import GameResult
from .options.GameResultOptions import GameResultOptions
from .SignedBallot import SignedBallot
from .options.SignedBallotOptions import SignedBallotOptions
from .WrittenBallot import WrittenBallot
from .options.WrittenBallotOptions import WrittenBallotOptions
from .Vote import Vote
from .options.VoteOptions import VoteOptions
from .CurrentMasterData import CurrentMasterData


# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *
from .options.AccessLogWithTelemetryOptions import AccessLogWithTelemetryOptions
from .enum.AccessLogWithTelemetryStatus import AccessLogWithTelemetryStatus


class AccessLogWithTelemetry:
    timestamp: int
    source_request_id: str
    request_id: str
    duration: int
    service: str
    method: str
    request: str
    result: str
    status: AccessLogWithTelemetryStatus
    user_id: Optional[str] = None

    def __init__(
        self,
        timestamp: int,
        source_request_id: str,
        request_id: str,
        duration: int,
        service: str,
        method: str,
        request: str,
        result: str,
        status: AccessLogWithTelemetryStatus,
        options: Optional[AccessLogWithTelemetryOptions] = AccessLogWithTelemetryOptions(),
    ):
        self.timestamp = timestamp
        self.source_request_id = source_request_id
        self.request_id = request_id
        self.duration = duration
        self.service = service
        self.method = method
        self.request = request
        self.result = result
        self.status = status
        self.user_id = options.user_id if options.user_id else None

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.timestamp is not None:
            properties["timestamp"] = self.timestamp
        if self.source_request_id is not None:
            properties["sourceRequestId"] = self.source_request_id
        if self.request_id is not None:
            properties["requestId"] = self.request_id
        if self.duration is not None:
            properties["duration"] = self.duration
        if self.service is not None:
            properties["service"] = self.service
        if self.method is not None:
            properties["method"] = self.method
        if self.user_id is not None:
            properties["userId"] = self.user_id
        if self.request is not None:
            properties["request"] = self.request
        if self.result is not None:
            properties["result"] = self.result
        if self.status is not None:
            properties["status"] = self.status.value

        return properties



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *

from ...core.model import CdkResource, Stack
from ...core.func import GetAttr
from ...core.model import NotificationSetting
from ...core.model import ScriptSetting
from ...core.model import LogSetting

from ..ref.NamespaceRef import NamespaceRef
from .CurrentMasterData import CurrentMasterData
from .GuildModel import GuildModel

from .options.NamespaceOptions import NamespaceOptions


class Namespace(CdkResource):
    stack: Stack
    name: str
    description: Optional[str] = None
    join_notification: Optional[NotificationSetting] = None
    leave_notification: Optional[NotificationSetting] = None
    change_member_notification: Optional[NotificationSetting] = None
    receive_request_notification: Optional[NotificationSetting] = None
    remove_request_notification: Optional[NotificationSetting] = None
    create_guild_script: Optional[ScriptSetting] = None
    join_guild_script: Optional[ScriptSetting] = None
    leave_guild_script: Optional[ScriptSetting] = None
    change_role_script: Optional[ScriptSetting] = None
    log_setting: Optional[LogSetting] = None

    def __init__(
        self,
        stack: Stack,
        name: str,
        options: Optional[NamespaceOptions] = NamespaceOptions(),
    ):
        super().__init__(
            "Guild_Namespace_" + name
        )

        self.stack = stack
        self.name = name
        self.description = options.description if options.description else None
        self.join_notification = options.join_notification if options.join_notification else None
        self.leave_notification = options.leave_notification if options.leave_notification else None
        self.change_member_notification = options.change_member_notification if options.change_member_notification else None
        self.receive_request_notification = options.receive_request_notification if options.receive_request_notification else None
        self.remove_request_notification = options.remove_request_notification if options.remove_request_notification else None
        self.create_guild_script = options.create_guild_script if options.create_guild_script else None
        self.join_guild_script = options.join_guild_script if options.join_guild_script else None
        self.leave_guild_script = options.leave_guild_script if options.leave_guild_script else None
        self.change_role_script = options.change_role_script if options.change_role_script else None
        self.log_setting = options.log_setting if options.log_setting else None
        stack.add_resource(
            self,
        )


    def alternate_keys(
        self,
    ):
        return "name"

    def resource_type(
        self,
    ) -> str:
        return "GS2::Guild::Namespace"

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.name is not None:
            properties["Name"] = self.name
        if self.description is not None:
            properties["Description"] = self.description
        if self.join_notification is not None:
            properties["JoinNotification"] = self.join_notification.properties(
            )
        if self.leave_notification is not None:
            properties["LeaveNotification"] = self.leave_notification.properties(
            )
        if self.change_member_notification is not None:
            properties["ChangeMemberNotification"] = self.change_member_notification.properties(
            )
        if self.receive_request_notification is not None:
            properties["ReceiveRequestNotification"] = self.receive_request_notification.properties(
            )
        if self.remove_request_notification is not None:
            properties["RemoveRequestNotification"] = self.remove_request_notification.properties(
            )
        if self.create_guild_script is not None:
            properties["CreateGuildScript"] = self.create_guild_script.properties(
            )
        if self.join_guild_script is not None:
            properties["JoinGuildScript"] = self.join_guild_script.properties(
            )
        if self.leave_guild_script is not None:
            properties["LeaveGuildScript"] = self.leave_guild_script.properties(
            )
        if self.change_role_script is not None:
            properties["ChangeRoleScript"] = self.change_role_script.properties(
            )
        if self.log_setting is not None:
            properties["LogSetting"] = self.log_setting.properties(
            )

        return properties

    def ref(
        self,
    ) -> NamespaceRef:
        return NamespaceRef(
            self.name,
        )

    def get_attr_namespace_id(
        self,
    ) -> GetAttr:
        return GetAttr(
            self,
            "Item.NamespaceId",
            None,
        )

    def master_data(
        self,
        guild_models: List[GuildModel],
    ) -> Namespace:
        CurrentMasterData(
            self.stack,
            self.name,
            guild_models,
        ).add_depends_on(
            self,
        )
        return self



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *
from .options.RepeatSettingOptions import RepeatSettingOptions
from .options.RepeatSettingRepeatTypeIsAlwaysOptions import RepeatSettingRepeatTypeIsAlwaysOptions
from .options.RepeatSettingRepeatTypeIsDailyOptions import RepeatSettingRepeatTypeIsDailyOptions
from .options.RepeatSettingRepeatTypeIsWeeklyOptions import RepeatSettingRepeatTypeIsWeeklyOptions
from .options.RepeatSettingRepeatTypeIsMonthlyOptions import RepeatSettingRepeatTypeIsMonthlyOptions
from .options.RepeatSettingRepeatTypeIsCustomOptions import RepeatSettingRepeatTypeIsCustomOptions
from .enum.RepeatSettingRepeatType import RepeatSettingRepeatType
from .enum.RepeatSettingBeginDayOfWeek import RepeatSettingBeginDayOfWeek
from .enum.RepeatSettingEndDayOfWeek import RepeatSettingEndDayOfWeek


class RepeatSetting:
    repeat_type: RepeatSettingRepeatType
    begin_day_of_month: Optional[int] = None
    end_day_of_month: Optional[int] = None
    begin_day_of_week: Optional[RepeatSettingBeginDayOfWeek] = None
    end_day_of_week: Optional[RepeatSettingEndDayOfWeek] = None
    begin_hour: Optional[int] = None
    end_hour: Optional[int] = None
    anchor_timestamp: Optional[int] = None
    active_days: Optional[int] = None
    inactive_days: Optional[int] = None

    def __init__(
        self,
        repeat_type: RepeatSettingRepeatType,
        options: Optional[RepeatSettingOptions] = RepeatSettingOptions(),
    ):
        self.repeat_type = repeat_type
        self.begin_day_of_month = options.begin_day_of_month if options.begin_day_of_month else None
        self.end_day_of_month = options.end_day_of_month if options.end_day_of_month else None
        self.begin_day_of_week = options.begin_day_of_week if options.begin_day_of_week else None
        self.end_day_of_week = options.end_day_of_week if options.end_day_of_week else None
        self.begin_hour = options.begin_hour if options.begin_hour else None
        self.end_hour = options.end_hour if options.end_hour else None
        self.anchor_timestamp = options.anchor_timestamp if options.anchor_timestamp else None
        self.active_days = options.active_days if options.active_days else None
        self.inactive_days = options.inactive_days if options.inactive_days else None

    @staticmethod
    def repeat_type_is_always(
        options: Optional[RepeatSettingRepeatTypeIsAlwaysOptions] = RepeatSettingRepeatTypeIsAlwaysOptions(),
    ) -> RepeatSetting:
        return RepeatSetting(
            RepeatSettingRepeatType.ALWAYS,
            RepeatSettingOptions(
            ),
        )

    @staticmethod
    def repeat_type_is_daily(
        begin_hour: int,
        end_hour: int,
        options: Optional[RepeatSettingRepeatTypeIsDailyOptions] = RepeatSettingRepeatTypeIsDailyOptions(),
    ) -> RepeatSetting:
        return RepeatSetting(
            RepeatSettingRepeatType.DAILY,
            RepeatSettingOptions(
                begin_hour,
                end_hour,
            ),
        )

    @staticmethod
    def repeat_type_is_weekly(
        begin_day_of_week: RepeatSettingBeginDayOfWeek,
        end_day_of_week: RepeatSettingEndDayOfWeek,
        begin_hour: int,
        end_hour: int,
        options: Optional[RepeatSettingRepeatTypeIsWeeklyOptions] = RepeatSettingRepeatTypeIsWeeklyOptions(),
    ) -> RepeatSetting:
        return RepeatSetting(
            RepeatSettingRepeatType.WEEKLY,
            RepeatSettingOptions(
                begin_day_of_week,
                end_day_of_week,
                begin_hour,
                end_hour,
            ),
        )

    @staticmethod
    def repeat_type_is_monthly(
        begin_day_of_month: int,
        end_day_of_month: int,
        begin_hour: int,
        end_hour: int,
        options: Optional[RepeatSettingRepeatTypeIsMonthlyOptions] = RepeatSettingRepeatTypeIsMonthlyOptions(),
    ) -> RepeatSetting:
        return RepeatSetting(
            RepeatSettingRepeatType.MONTHLY,
            RepeatSettingOptions(
                begin_day_of_month,
                end_day_of_month,
                begin_hour,
                end_hour,
            ),
        )

    @staticmethod
    def repeat_type_is_custom(
        anchor_timestamp: int,
        active_days: int,
        inactive_days: int,
        options: Optional[RepeatSettingRepeatTypeIsCustomOptions] = RepeatSettingRepeatTypeIsCustomOptions(),
    ) -> RepeatSetting:
        return RepeatSetting(
            RepeatSettingRepeatType.CUSTOM,
            RepeatSettingOptions(
                anchor_timestamp,
                active_days,
                inactive_days,
            ),
        )

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.repeat_type is not None:
            properties["repeatType"] = self.repeat_type.value
        if self.begin_day_of_month is not None:
            properties["beginDayOfMonth"] = self.begin_day_of_month
        if self.end_day_of_month is not None:
            properties["endDayOfMonth"] = self.end_day_of_month
        if self.begin_day_of_week is not None:
            properties["beginDayOfWeek"] = self.begin_day_of_week.value
        if self.end_day_of_week is not None:
            properties["endDayOfWeek"] = self.end_day_of_week.value
        if self.begin_hour is not None:
            properties["beginHour"] = self.begin_hour
        if self.end_hour is not None:
            properties["endHour"] = self.end_hour
        if self.anchor_timestamp is not None:
            properties["anchorTimestamp"] = self.anchor_timestamp
        if self.active_days is not None:
            properties["activeDays"] = self.active_days
        if self.inactive_days is not None:
            properties["inactiveDays"] = self.inactive_days

        return properties



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *
from .Version import Version
from .ScheduleVersion import ScheduleVersion
from .VersionModel import VersionModel
from .options.StatusOptions import StatusOptions


class Status:
    version_model: VersionModel
    current_version: Optional[Version] = None

    def __init__(
        self,
        version_model: VersionModel,
        options: Optional[StatusOptions] = StatusOptions(),
    ):
        self.version_model = version_model
        self.current_version = options.current_version if options.current_version else None

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.version_model is not None:
            properties["versionModel"] = self.version_model.properties(
            )
        if self.current_version is not None:
            properties["currentVersion"] = self.current_version.properties(
            )

        return properties



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *
from .DepositTransaction import DepositTransaction
from .WalletSummary import WalletSummary
from .options.DepositEventOptions import DepositEventOptions


class DepositEvent:
    slot: int
    status: WalletSummary
    deposit_transactions: Optional[List[DepositTransaction]] = None

    def __init__(
        self,
        slot: int,
        status: WalletSummary,
        options: Optional[DepositEventOptions] = DepositEventOptions(),
    ):
        self.slot = slot
        self.status = status
        self.deposit_transactions = options.deposit_transactions if options.deposit_transactions else None

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.slot is not None:
            properties["slot"] = self.slot
        if self.deposit_transactions is not None:
            properties["depositTransactions"] = [
                v.properties(
                )
                for v in self.deposit_transactions
            ]
        if self.status is not None:
            properties["status"] = self.status.properties(
            )

        return properties



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from .Namespace import Namespace
from .options.NamespaceOptions import NamespaceOptions
from .RateModel import RateModel
from .options.RateModelOptions import RateModelOptions
from .UnleashRateModel import UnleashRateModel
from .options.UnleashRateModelOptions import UnleashRateModelOptions
from .BonusRate import BonusRate
from .options.BonusRateOptions import BonusRateOptions
from .Material import Material
from .options.MaterialOptions import MaterialOptions
from .UnleashRateEntryModel import UnleashRateEntryModel
from .options.UnleashRateEntryModelOptions import UnleashRateEntryModelOptions
from .CurrentMasterData import CurrentMasterData


# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *

from ...core.model import CdkResource, Stack
from .GlobalMessage import GlobalMessage


class CurrentMasterData(CdkResource):
    version: str= "2020-03-12"
    namespace_name: str
    global_messages: List[GlobalMessage]

    def __init__(
        self,
        stack: Stack,
        namespace_name: str,
        global_messages: List[GlobalMessage],
    ):
        super().__init__(
            "Inbox_CurrentMessageMaster_" + namespace_name
        )

        self.namespace_name = namespace_name
        self.global_messages = global_messages
        stack.add_resource(
            self,
        )

    def alternate_keys(
        self,
    ):
        return self.namespace_name

    def resource_type(
        self,
    ) -> str:
        return "GS2::Inbox::CurrentMessageMaster"

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}
        settings: Dict[str, Any] = {}

        settings["version"] = self.version
        if self.global_messages is not None:
            settings["globalMessages"] = [
                v.properties(
                )
                for v in self.global_messages
            ]

        if self.namespace_name is not None:
            properties["NamespaceName"] = self.namespace_name
        if settings is not None:
            properties["Settings"] = settings

        return properties


# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *
from .options.LogCostOptions import LogCostOptions


class LogCost:
    base: float
    adds: List[float]
    subs: Optional[List[float]] = None

    def __init__(
        self,
        base: float,
        adds: List[float],
        options: Optional[LogCostOptions] = LogCostOptions(),
    ):
        self.base = base
        self.adds = adds
        self.subs = options.subs if options.subs else None

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.base is not None:
            properties["base"] = self.base
        if self.adds is not None:
            properties["adds"] = self.adds
        if self.subs is not None:
            properties["subs"] = self.subs

        return properties



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from .Namespace import Namespace
from .options.NamespaceOptions import NamespaceOptions
from .GlobalRankingModel import GlobalRankingModel
from .options.GlobalRankingModelOptions import GlobalRankingModelOptions
from .enum.GlobalRankingModelOrderDirection import GlobalRankingModelOrderDirection
from .ClusterRankingModel import ClusterRankingModel
from .options.ClusterRankingModelOptions import ClusterRankingModelOptions
from .enum.ClusterRankingModelClusterType import ClusterRankingModelClusterType
from .enum.ClusterRankingModelOrderDirection import ClusterRankingModelOrderDirection
from .SubscribeRankingModel import SubscribeRankingModel
from .options.SubscribeRankingModelOptions import SubscribeRankingModelOptions
from .enum.SubscribeRankingModelOrderDirection import SubscribeRankingModelOrderDirection
from .RankingReward import RankingReward
from .options.RankingRewardOptions import RankingRewardOptions
from .CurrentMasterData import CurrentMasterData


# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *
from .options.SubscribeRankingModelOptions import SubscribeRankingModelOptions
from .enum.SubscribeRankingModelOrderDirection import SubscribeRankingModelOrderDirection


class SubscribeRankingModel:
    name: str
    sum: bool
    order_direction: SubscribeRankingModelOrderDirection
    metadata: Optional[str] = None
    minimum_value: Optional[int] = None
    maximum_value: Optional[int] = None
    entry_period_event_id: Optional[str] = None
    access_period_event_id: Optional[str] = None

    def __init__(
        self,
        name: str,
        sum: bool,
        order_direction: SubscribeRankingModelOrderDirection,
        options: Optional[SubscribeRankingModelOptions] = SubscribeRankingModelOptions(),
    ):
        self.name = name
        self.sum = sum
        self.order_direction = order_direction
        self.metadata = options.metadata if options.metadata else None
        self.minimum_value = options.minimum_value if options.minimum_value else None
        self.maximum_value = options.maximum_value if options.maximum_value else None
        self.entry_period_event_id = options.entry_period_event_id if options.entry_period_event_id else None
        self.access_period_event_id = options.access_period_event_id if options.access_period_event_id else None

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.name is not None:
            properties["name"] = self.name
        if self.metadata is not None:
            properties["metadata"] = self.metadata
        if self.minimum_value is not None:
            properties["minimumValue"] = self.minimum_value
        if self.maximum_value is not None:
            properties["maximumValue"] = self.maximum_value
        if self.sum is not None:
            properties["sum"] = self.sum
        if self.order_direction is not None:
            properties["orderDirection"] = self.order_direction.value
        if self.entry_period_event_id is not None:
            properties["entryPeriodEventId"] = self.entry_period_event_id
        if self.access_period_event_id is not None:
            properties["accessPeriodEventId"] = self.access_period_event_id

        return properties



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from .AddCapacityByUserId import AddCapacityByUserId
from .SetCapacityByUserId import SetCapacityByUserId
from .AcquireItemSetByUserId import AcquireItemSetByUserId
from .AcquireItemSetWithGradeByUserId import AcquireItemSetWithGradeByUserId
from .AddReferenceOfByUserId import AddReferenceOfByUserId
from .DeleteReferenceOfByUserId import DeleteReferenceOfByUserId
from .AcquireSimpleItemsByUserId import AcquireSimpleItemsByUserId
from .SetSimpleItemsByUserId import SetSimpleItemsByUserId
from .AcquireBigItemByUserId import AcquireBigItemByUserId
from .SetBigItemByUserId import SetBigItemByUserId
from .ConsumeItemSetByUserId import ConsumeItemSetByUserId
from .ConsumeSimpleItemsByUserId import ConsumeSimpleItemsByUserId
from .ConsumeBigItemByUserId import ConsumeBigItemByUserId
from .VerifyInventoryCurrentMaxCapacityByUserId import VerifyInventoryCurrentMaxCapacityByUserId
from .VerifyItemSetByUserId import VerifyItemSetByUserId
from .VerifyReferenceOfByUserId import VerifyReferenceOfByUserId
from .VerifySimpleItemByUserId import VerifySimpleItemByUserId
from .VerifyBigItemByUserId import VerifyBigItemByUserId


# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *
from ...core.model import AcquireAction
from .Reward import Reward
from ...core.model import VerifyAction
from ...core.model import ConsumeAction
from .options.BonusModelOptions import BonusModelOptions
from .options.BonusModelModeIsScheduleOptions import BonusModelModeIsScheduleOptions
from .options.BonusModelModeIsStreamingOptions import BonusModelModeIsStreamingOptions
from .options.BonusModelMissedReceiveReliefIsEnabledOptions import BonusModelMissedReceiveReliefIsEnabledOptions
from .options.BonusModelMissedReceiveReliefIsDisabledOptions import BonusModelMissedReceiveReliefIsDisabledOptions
from .enum.BonusModelMode import BonusModelMode
from .enum.BonusModelRepeat import BonusModelRepeat
from .enum.BonusModelMissedReceiveRelief import BonusModelMissedReceiveRelief


class BonusModel:
    name: str
    mode: BonusModelMode
    missed_receive_relief: BonusModelMissedReceiveRelief
    metadata: Optional[str] = None
    period_event_id: Optional[str] = None
    reset_hour: Optional[int] = None
    repeat: Optional[BonusModelRepeat] = None
    rewards: Optional[List[Reward]] = None
    missed_receive_relief_verify_actions: Optional[List[VerifyAction]] = None
    missed_receive_relief_consume_actions: Optional[List[ConsumeAction]] = None

    def __init__(
        self,
        name: str,
        mode: BonusModelMode,
        missed_receive_relief: BonusModelMissedReceiveRelief,
        options: Optional[BonusModelOptions] = BonusModelOptions(),
    ):
        self.name = name
        self.mode = mode
        self.missed_receive_relief = missed_receive_relief
        self.metadata = options.metadata if options.metadata else None
        self.period_event_id = options.period_event_id if options.period_event_id else None
        self.reset_hour = options.reset_hour if options.reset_hour else None
        self.repeat = options.repeat if options.repeat else None
        self.rewards = options.rewards if options.rewards else None
        self.missed_receive_relief_verify_actions = options.missed_receive_relief_verify_actions if options.missed_receive_relief_verify_actions else None
        self.missed_receive_relief_consume_actions = options.missed_receive_relief_consume_actions if options.missed_receive_relief_consume_actions else None

    @staticmethod
    def mode_is_schedule(
        name: str,
        missed_receive_relief: BonusModelMissedReceiveRelief,
        options: Optional[BonusModelModeIsScheduleOptions] = BonusModelModeIsScheduleOptions(),
    ) -> BonusModel:
        return BonusModel(
            name,
            BonusModelMode.SCHEDULE,
            missed_receive_relief,
            BonusModelOptions(
                options.metadata,
                options.period_event_id,
                options.rewards,
                options.missed_receive_relief_verify_actions,
                options.missed_receive_relief_consume_actions,
            ),
        )

    @staticmethod
    def mode_is_streaming(
        name: str,
        missed_receive_relief: BonusModelMissedReceiveRelief,
        repeat: BonusModelRepeat,
        options: Optional[BonusModelModeIsStreamingOptions] = BonusModelModeIsStreamingOptions(),
    ) -> BonusModel:
        return BonusModel(
            name,
            BonusModelMode.STREAMING,
            missed_receive_relief,
            BonusModelOptions(
                repeat,
                options.metadata,
                options.period_event_id,
                options.rewards,
                options.missed_receive_relief_verify_actions,
                options.missed_receive_relief_consume_actions,
            ),
        )

    @staticmethod
    def missed_receive_relief_is_enabled(
        name: str,
        mode: BonusModelMode,
        options: Optional[BonusModelMissedReceiveReliefIsEnabledOptions] = BonusModelMissedReceiveReliefIsEnabledOptions(),
    ) -> BonusModel:
        return BonusModel(
            name,
            mode,
            BonusModelMissedReceiveRelief.ENABLED,
            BonusModelOptions(
                options.metadata,
                options.period_event_id,
                options.rewards,
                options.missed_receive_relief_verify_actions,
                options.missed_receive_relief_consume_actions,
            ),
        )

    @staticmethod
    def missed_receive_relief_is_disabled(
        name: str,
        mode: BonusModelMode,
        options: Optional[BonusModelMissedReceiveReliefIsDisabledOptions] = BonusModelMissedReceiveReliefIsDisabledOptions(),
    ) -> BonusModel:
        return BonusModel(
            name,
            mode,
            BonusModelMissedReceiveRelief.DISABLED,
            BonusModelOptions(
                options.metadata,
                options.period_event_id,
                options.rewards,
                options.missed_receive_relief_verify_actions,
                options.missed_receive_relief_consume_actions,
            ),
        )

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.name is not None:
            properties["name"] = self.name
        if self.metadata is not None:
            properties["metadata"] = self.metadata
        if self.mode is not None:
            properties["mode"] = self.mode.value
        if self.period_event_id is not None:
            properties["periodEventId"] = self.period_event_id
        if self.reset_hour is not None:
            properties["resetHour"] = self.reset_hour
        if self.repeat is not None:
            properties["repeat"] = self.repeat.value
        if self.rewards is not None:
            properties["rewards"] = [
                v.properties(
                )
                for v in self.rewards
            ]
        if self.missed_receive_relief is not None:
            properties["missedReceiveRelief"] = self.missed_receive_relief.value
        if self.missed_receive_relief_verify_actions is not None:
            properties["missedReceiveReliefVerifyActions"] = [
                v.properties(
                )
                for v in self.missed_receive_relief_verify_actions
            ]
        if self.missed_receive_relief_consume_actions is not None:
            properties["missedReceiveReliefConsumeActions"] = [
                v.properties(
                )
                for v in self.missed_receive_relief_consume_actions
            ]

        return properties



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *
from .TargetCounterModel import TargetCounterModel
from ...core.model import VerifyAction
from ...core.model import AcquireAction
from .options.MissionTaskModelOptions import MissionTaskModelOptions
from .options.MissionTaskModelVerifyCompleteTypeIsCounterOptions import MissionTaskModelVerifyCompleteTypeIsCounterOptions
from .options.MissionTaskModelVerifyCompleteTypeIsVerifyActionsOptions import MissionTaskModelVerifyCompleteTypeIsVerifyActionsOptions
from .enum.MissionTaskModelVerifyCompleteType import MissionTaskModelVerifyCompleteType
from .enum.MissionTaskModelTargetResetType import MissionTaskModelTargetResetType


class MissionTaskModel:
    name: str
    verify_complete_type: MissionTaskModelVerifyCompleteType
    counter_name: str
    target_value: int
    metadata: Optional[str] = None
    target_counter: Optional[TargetCounterModel] = None
    verify_complete_consume_actions: Optional[List[VerifyAction]] = None
    complete_acquire_actions: Optional[List[AcquireAction]] = None
    challenge_period_event_id: Optional[str] = None
    premise_mission_task_name: Optional[str] = None
    target_reset_type: Optional[MissionTaskModelTargetResetType] = None

    def __init__(
        self,
        name: str,
        verify_complete_type: MissionTaskModelVerifyCompleteType,
        counter_name: str,
        target_value: int,
        options: Optional[MissionTaskModelOptions] = MissionTaskModelOptions(),
    ):
        self.name = name
        self.verify_complete_type = verify_complete_type
        self.counter_name = counter_name
        self.target_value = target_value
        self.metadata = options.metadata if options.metadata else None
        self.target_counter = options.target_counter if options.target_counter else None
        self.verify_complete_consume_actions = options.verify_complete_consume_actions if options.verify_complete_consume_actions else None
        self.complete_acquire_actions = options.complete_acquire_actions if options.complete_acquire_actions else None
        self.challenge_period_event_id = options.challenge_period_event_id if options.challenge_period_event_id else None
        self.premise_mission_task_name = options.premise_mission_task_name if options.premise_mission_task_name else None
        self.target_reset_type = options.target_reset_type if options.target_reset_type else None

    @staticmethod
    def verify_complete_type_is_counter(
        name: str,
        counter_name: str,
        target_value: int,
        target_counter: TargetCounterModel,
        options: Optional[MissionTaskModelVerifyCompleteTypeIsCounterOptions] = MissionTaskModelVerifyCompleteTypeIsCounterOptions(),
    ) -> MissionTaskModel:
        return MissionTaskModel(
            name,
            MissionTaskModelVerifyCompleteType.COUNTER,
            counter_name,
            target_value,
            MissionTaskModelOptions(
                target_counter,
                options.metadata,
                options.verify_complete_consume_actions,
                options.complete_acquire_actions,
                options.challenge_period_event_id,
                options.premise_mission_task_name,
                options.target_reset_type,
            ),
        )

    @staticmethod
    def verify_complete_type_is_verify_actions(
        name: str,
        counter_name: str,
        target_value: int,
        options: Optional[MissionTaskModelVerifyCompleteTypeIsVerifyActionsOptions] = MissionTaskModelVerifyCompleteTypeIsVerifyActionsOptions(),
    ) -> MissionTaskModel:
        return MissionTaskModel(
            name,
            MissionTaskModelVerifyCompleteType.VERIFY_ACTIONS,
            counter_name,
            target_value,
            MissionTaskModelOptions(
                options.metadata,
                options.verify_complete_consume_actions,
                options.complete_acquire_actions,
                options.challenge_period_event_id,
                options.premise_mission_task_name,
                options.target_reset_type,
            ),
        )

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.name is not None:
            properties["name"] = self.name
        if self.metadata is not None:
            properties["metadata"] = self.metadata
        if self.verify_complete_type is not None:
            properties["verifyCompleteType"] = self.verify_complete_type.value
        if self.target_counter is not None:
            properties["targetCounter"] = self.target_counter.properties(
            )
        if self.verify_complete_consume_actions is not None:
            properties["verifyCompleteConsumeActions"] = [
                v.properties(
                )
                for v in self.verify_complete_consume_actions
            ]
        if self.complete_acquire_actions is not None:
            properties["completeAcquireActions"] = [
                v.properties(
                )
                for v in self.complete_acquire_actions
            ]
        if self.challenge_period_event_id is not None:
            properties["challengePeriodEventId"] = self.challenge_period_event_id
        if self.premise_mission_task_name is not None:
            properties["premiseMissionTaskName"] = self.premise_mission_task_name
        if self.counter_name is not None:
            properties["counterName"] = self.counter_name
        if self.target_reset_type is not None:
            properties["targetResetType"] = self.target_reset_type.value
        if self.target_value is not None:
            properties["targetValue"] = self.target_value

        return properties



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *
from .BuffTargetGrn import BuffTargetGrn
from .options.BuffTargetModelOptions import BuffTargetModelOptions
from .enum.BuffTargetModelTargetModelName import BuffTargetModelTargetModelName


class BuffTargetModel:
    target_model_name: str
    target_field_name: str
    condition_grns: List[BuffTargetGrn]
    rate: float

    def __init__(
        self,
        target_model_name: str,
        target_field_name: str,
        condition_grns: List[BuffTargetGrn],
        rate: float,
        options: Optional[BuffTargetModelOptions] = BuffTargetModelOptions(),
    ):
        self.target_model_name = target_model_name
        self.target_field_name = target_field_name
        self.condition_grns = condition_grns
        self.rate = rate

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.target_model_name is not None:
            properties["targetModelName"] = self.target_model_name.value
        if self.target_field_name is not None:
            properties["targetFieldName"] = self.target_field_name
        if self.condition_grns is not None:
            properties["conditionGrns"] = [
                v.properties(
                )
                for v in self.condition_grns
            ]
        if self.rate is not None:
            properties["rate"] = self.rate

        return properties



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *
from .BuffTargetGrn import BuffTargetGrn
from .BuffTargetModel import BuffTargetModel
from .BuffTargetAction import BuffTargetAction
from .options.BuffEntryModelOptions import BuffEntryModelOptions
from .options.BuffEntryModelTargetTypeIsModelOptions import BuffEntryModelTargetTypeIsModelOptions
from .options.BuffEntryModelTargetTypeIsActionOptions import BuffEntryModelTargetTypeIsActionOptions
from .enum.BuffEntryModelExpression import BuffEntryModelExpression
from .enum.BuffEntryModelTargetType import BuffEntryModelTargetType


class BuffEntryModel:
    name: str
    expression: BuffEntryModelExpression
    target_type: BuffEntryModelTargetType
    priority: int
    metadata: Optional[str] = None
    target_model: Optional[BuffTargetModel] = None
    target_action: Optional[BuffTargetAction] = None
    apply_period_schedule_event_id: Optional[str] = None

    def __init__(
        self,
        name: str,
        expression: BuffEntryModelExpression,
        target_type: BuffEntryModelTargetType,
        priority: int,
        options: Optional[BuffEntryModelOptions] = BuffEntryModelOptions(),
    ):
        self.name = name
        self.expression = expression
        self.target_type = target_type
        self.priority = priority
        self.metadata = options.metadata if options.metadata else None
        self.target_model = options.target_model if options.target_model else None
        self.target_action = options.target_action if options.target_action else None
        self.apply_period_schedule_event_id = options.apply_period_schedule_event_id if options.apply_period_schedule_event_id else None

    @staticmethod
    def target_type_is_model(
        name: str,
        expression: BuffEntryModelExpression,
        priority: int,
        target_model: BuffTargetModel,
        options: Optional[BuffEntryModelTargetTypeIsModelOptions] = BuffEntryModelTargetTypeIsModelOptions(),
    ) -> BuffEntryModel:
        return BuffEntryModel(
            name,
            expression,
            BuffEntryModelTargetType.MODEL,
            priority,
            BuffEntryModelOptions(
                target_model,
                options.metadata,
                options.apply_period_schedule_event_id,
            ),
        )

    @staticmethod
    def target_type_is_action(
        name: str,
        expression: BuffEntryModelExpression,
        priority: int,
        target_action: BuffTargetAction,
        options: Optional[BuffEntryModelTargetTypeIsActionOptions] = BuffEntryModelTargetTypeIsActionOptions(),
    ) -> BuffEntryModel:
        return BuffEntryModel(
            name,
            expression,
            BuffEntryModelTargetType.ACTION,
            priority,
            BuffEntryModelOptions(
                target_action,
                options.metadata,
                options.apply_period_schedule_event_id,
            ),
        )

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.name is not None:
            properties["name"] = self.name
        if self.metadata is not None:
            properties["metadata"] = self.metadata
        if self.expression is not None:
            properties["expression"] = self.expression.value
        if self.target_type is not None:
            properties["targetType"] = self.target_type.value
        if self.target_model is not None:
            properties["targetModel"] = self.target_model.properties(
            )
        if self.target_action is not None:
            properties["targetAction"] = self.target_action.properties(
            )
        if self.priority is not None:
            properties["priority"] = self.priority
        if self.apply_period_schedule_event_id is not None:
            properties["applyPeriodScheduleEventId"] = self.apply_period_schedule_event_id

        return properties



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *
from .options.DailyTransactionHistoryOptions import DailyTransactionHistoryOptions


class DailyTransactionHistory:
    year: int
    month: int
    day: int
    currency: str
    deposit_amount: float
    withdraw_amount: float
    revision: Optional[int] = None

    def __init__(
        self,
        year: int,
        month: int,
        day: int,
        currency: str,
        deposit_amount: float,
        withdraw_amount: float,
        options: Optional[DailyTransactionHistoryOptions] = DailyTransactionHistoryOptions(),
    ):
        self.year = year
        self.month = month
        self.day = day
        self.currency = currency
        self.deposit_amount = deposit_amount
        self.withdraw_amount = withdraw_amount
        self.revision = options.revision if options.revision else None

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.year is not None:
            properties["year"] = self.year
        if self.month is not None:
            properties["month"] = self.month
        if self.day is not None:
            properties["day"] = self.day
        if self.currency is not None:
            properties["currency"] = self.currency
        if self.deposit_amount is not None:
            properties["depositAmount"] = self.deposit_amount
        if self.withdraw_amount is not None:
            properties["withdrawAmount"] = self.withdraw_amount

        return properties



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *
from .options.RewardOptions import RewardOptions


class Reward:
    action: str
    request: str
    item_id: str
    value: int

    def __init__(
        self,
        action: str,
        request: str,
        item_id: str,
        value: int,
        options: Optional[RewardOptions] = RewardOptions(),
    ):
        self.action = action
        self.request = request
        self.item_id = item_id
        self.value = value

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.action is not None:
            properties["action"] = self.action.value
        if self.request is not None:
            properties["request"] = self.request
        if self.item_id is not None:
            properties["itemId"] = self.item_id
        if self.value is not None:
            properties["value"] = self.value

        return properties



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *

from ...core.func import GetAttr, Join


class UnusedBalanceRef:
    namespace_name: str
    currency: str

    def __init__(
        self,
        namespace_name: str,
        currency: str,
    ):
        self.namespace_name = namespace_name
        self.currency = currency

    def grn(
        self,
    ) -> str:
        return Join(
            ":",
            [
                "grn",
                "gs2",
                GetAttr.region(
                ).str(
                ),
                GetAttr.owner_id(
                ).str(
                ),
                "money2",
                self.namespace_name,
                "unused",
                self.currency,
            ],
        ).str(
        )



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from .Namespace import Namespace
from .options.NamespaceOptions import NamespaceOptions
from .StaminaModel import StaminaModel
from .options.StaminaModelOptions import StaminaModelOptions
from .MaxStaminaTable import MaxStaminaTable
from .options.MaxStaminaTableOptions import MaxStaminaTableOptions
from .RecoverIntervalTable import RecoverIntervalTable
from .options.RecoverIntervalTableOptions import RecoverIntervalTableOptions
from .RecoverValueTable import RecoverValueTable
from .options.RecoverValueTableOptions import RecoverValueTableOptions
from .CurrentMasterData import CurrentMasterData


# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *
from .options.ScopedValueOptions import ScopedValueOptions
from .options.ScopedValueScopeTypeIsResetTimingOptions import ScopedValueScopeTypeIsResetTimingOptions
from .options.ScopedValueScopeTypeIsVerifyActionOptions import ScopedValueScopeTypeIsVerifyActionOptions
from .enum.ScopedValueScopeType import ScopedValueScopeType
from .enum.ScopedValueResetType import ScopedValueResetType


class ScopedValue:
    scope_type: ScopedValueScopeType
    value: int
    reset_type: Optional[ScopedValueResetType] = None
    condition_name: Optional[str] = None
    next_reset_at: Optional[int] = None

    def __init__(
        self,
        scope_type: ScopedValueScopeType,
        value: int,
        options: Optional[ScopedValueOptions] = ScopedValueOptions(),
    ):
        self.scope_type = scope_type
        self.value = value
        self.reset_type = options.reset_type if options.reset_type else None
        self.condition_name = options.condition_name if options.condition_name else None
        self.next_reset_at = options.next_reset_at if options.next_reset_at else None

    @staticmethod
    def scope_type_is_reset_timing(
        value: int,
        updated_at: int,
        reset_type: ScopedValueResetType,
        options: Optional[ScopedValueScopeTypeIsResetTimingOptions] = ScopedValueScopeTypeIsResetTimingOptions(),
    ) -> ScopedValue:
        return ScopedValue(
            ScopedValueScopeType.RESET_TIMING,
            value,
            updated_at,
            ScopedValueOptions(
                reset_type,
                options.next_reset_at,
            ),
        )

    @staticmethod
    def scope_type_is_verify_action(
        value: int,
        updated_at: int,
        condition_name: str,
        options: Optional[ScopedValueScopeTypeIsVerifyActionOptions] = ScopedValueScopeTypeIsVerifyActionOptions(),
    ) -> ScopedValue:
        return ScopedValue(
            ScopedValueScopeType.VERIFY_ACTION,
            value,
            updated_at,
            ScopedValueOptions(
                condition_name,
                options.next_reset_at,
            ),
        )

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.scope_type is not None:
            properties["scopeType"] = self.scope_type.value
        if self.reset_type is not None:
            properties["resetType"] = self.reset_type.value
        if self.condition_name is not None:
            properties["conditionName"] = self.condition_name
        if self.value is not None:
            properties["value"] = self.value
        if self.next_reset_at is not None:
            properties["nextResetAt"] = self.next_reset_at

        return properties



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from .Namespace import Namespace
from .options.NamespaceOptions import NamespaceOptions
from .RateModel import RateModel
from .options.RateModelOptions import RateModelOptions
from .enum.RateModelTimingType import RateModelTimingType
from .options.RateModelTimingTypeIsImmediateOptions import RateModelTimingTypeIsImmediateOptions
from .options.RateModelTimingTypeIsAwaitOptions import RateModelTimingTypeIsAwaitOptions
from .IncrementalRateModel import IncrementalRateModel
from .options.IncrementalRateModelOptions import IncrementalRateModelOptions
from .enum.IncrementalRateModelCalculateType import IncrementalRateModelCalculateType
from .options.IncrementalRateModelCalculateTypeIsLinearOptions import IncrementalRateModelCalculateTypeIsLinearOptions
from .options.IncrementalRateModelCalculateTypeIsPowerOptions import IncrementalRateModelCalculateTypeIsPowerOptions
from .options.IncrementalRateModelCalculateTypeIsGs2ScriptOptions import IncrementalRateModelCalculateTypeIsGs2ScriptOptions
from .LogCost import LogCost
from .options.LogCostOptions import LogCostOptions
from .CurrentMasterData import CurrentMasterData


# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *

from ...core.model import CdkResource, Stack
from ...core.func import GetAttr
from ...core.model import ScriptSetting
from ...core.model import LogSetting

from ..ref.NamespaceRef import NamespaceRef
from .CurrentMasterData import CurrentMasterData
from .TakeOverTypeModel import TakeOverTypeModel

from .options.NamespaceOptions import NamespaceOptions


class Namespace(CdkResource):
    stack: Stack
    name: str
    description: Optional[str] = None
    change_password_if_take_over: Optional[bool] = None
    different_user_id_for_login_and_data_retention: Optional[bool] = None
    create_account_script: Optional[ScriptSetting] = None
    authentication_script: Optional[ScriptSetting] = None
    create_take_over_script: Optional[ScriptSetting] = None
    do_take_over_script: Optional[ScriptSetting] = None
    log_setting: Optional[LogSetting] = None

    def __init__(
        self,
        stack: Stack,
        name: str,
        options: Optional[NamespaceOptions] = NamespaceOptions(),
    ):
        super().__init__(
            "Account_Namespace_" + name
        )

        self.stack = stack
        self.name = name
        self.description = options.description if options.description else None
        self.change_password_if_take_over = options.change_password_if_take_over if options.change_password_if_take_over else None
        self.different_user_id_for_login_and_data_retention = options.different_user_id_for_login_and_data_retention if options.different_user_id_for_login_and_data_retention else None
        self.create_account_script = options.create_account_script if options.create_account_script else None
        self.authentication_script = options.authentication_script if options.authentication_script else None
        self.create_take_over_script = options.create_take_over_script if options.create_take_over_script else None
        self.do_take_over_script = options.do_take_over_script if options.do_take_over_script else None
        self.log_setting = options.log_setting if options.log_setting else None
        stack.add_resource(
            self,
        )


    def alternate_keys(
        self,
    ):
        return "name"

    def resource_type(
        self,
    ) -> str:
        return "GS2::Account::Namespace"

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.name is not None:
            properties["Name"] = self.name
        if self.description is not None:
            properties["Description"] = self.description
        if self.change_password_if_take_over is not None:
            properties["ChangePasswordIfTakeOver"] = self.change_password_if_take_over
        if self.different_user_id_for_login_and_data_retention is not None:
            properties["DifferentUserIdForLoginAndDataRetention"] = self.different_user_id_for_login_and_data_retention
        if self.create_account_script is not None:
            properties["CreateAccountScript"] = self.create_account_script.properties(
            )
        if self.authentication_script is not None:
            properties["AuthenticationScript"] = self.authentication_script.properties(
            )
        if self.create_take_over_script is not None:
            properties["CreateTakeOverScript"] = self.create_take_over_script.properties(
            )
        if self.do_take_over_script is not None:
            properties["DoTakeOverScript"] = self.do_take_over_script.properties(
            )
        if self.log_setting is not None:
            properties["LogSetting"] = self.log_setting.properties(
            )

        return properties

    def ref(
        self,
    ) -> NamespaceRef:
        return NamespaceRef(
            self.name,
        )

    def get_attr_namespace_id(
        self,
    ) -> GetAttr:
        return GetAttr(
            self,
            "Item.NamespaceId",
            None,
        )

    def master_data(
        self,
        take_over_type_models: List[TakeOverTypeModel],
    ) -> Namespace:
        CurrentMasterData(
            self.stack,
            self.name,
            take_over_type_models,
        ).add_depends_on(
            self,
        )
        return self



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *
from .Ballot import Ballot
from .GameResult import GameResult
from .WrittenBallot import WrittenBallot
from .options.VoteOptions import VoteOptions


class Vote:
    season_name: str
    session_name: str
    written_ballots: Optional[List[WrittenBallot]] = None
    revision: Optional[int] = None

    def __init__(
        self,
        season_name: str,
        session_name: str,
        options: Optional[VoteOptions] = VoteOptions(),
    ):
        self.season_name = season_name
        self.session_name = session_name
        self.written_ballots = options.written_ballots if options.written_ballots else None
        self.revision = options.revision if options.revision else None

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.season_name is not None:
            properties["seasonName"] = self.season_name
        if self.session_name is not None:
            properties["sessionName"] = self.session_name
        if self.written_ballots is not None:
            properties["writtenBallots"] = [
                v.properties(
                )
                for v in self.written_ballots
            ]

        return properties



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *
from .options.OpenIdConnectSettingOptions import OpenIdConnectSettingOptions


class OpenIdConnectSetting:
    configuration_path: str
    client_id: str
    client_secret: Optional[str] = None
    apple_team_id: Optional[str] = None
    apple_key_id: Optional[str] = None
    apple_private_key_pem: Optional[str] = None

    def __init__(
        self,
        configuration_path: str,
        client_id: str,
        options: Optional[OpenIdConnectSettingOptions] = OpenIdConnectSettingOptions(),
    ):
        self.configuration_path = configuration_path
        self.client_id = client_id
        self.client_secret = options.client_secret if options.client_secret else None
        self.apple_team_id = options.apple_team_id if options.apple_team_id else None
        self.apple_key_id = options.apple_key_id if options.apple_key_id else None
        self.apple_private_key_pem = options.apple_private_key_pem if options.apple_private_key_pem else None

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.configuration_path is not None:
            properties["configurationPath"] = self.configuration_path
        if self.client_id is not None:
            properties["clientId"] = self.client_id
        if self.client_secret is not None:
            properties["clientSecret"] = self.client_secret
        if self.apple_team_id is not None:
            properties["appleTeamId"] = self.apple_team_id
        if self.apple_key_id is not None:
            properties["appleKeyId"] = self.apple_key_id
        if self.apple_private_key_pem is not None:
            properties["applePrivateKeyPem"] = self.apple_private_key_pem

        return properties



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *

from ...core.model import CdkResource, Stack
from ...core.func import GetAttr
from .Policy import Policy

from ..ref.SecurityPolicyRef import SecurityPolicyRef

from .options.SecurityPolicyOptions import SecurityPolicyOptions


class SecurityPolicy(CdkResource):
    stack: Stack
    name: str
    policy: Policy
    description: Optional[str] = None

    def __init__(
        self,
        stack: Stack,
        name: str,
        policy: Policy,
        options: Optional[SecurityPolicyOptions] = SecurityPolicyOptions(),
    ):
        super().__init__(
            "Identifier_SecurityPolicy_" + name
        )

        self.stack = stack
        self.name = name
        self.policy = policy
        self.description = options.description if options.description else None
        stack.add_resource(
            self,
        )


    def alternate_keys(
        self,
    ):
        return "name"

    def resource_type(
        self,
    ) -> str:
        return "GS2::Identifier::SecurityPolicy"

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.name is not None:
            properties["Name"] = self.name
        if self.description is not None:
            properties["Description"] = self.description
        if self.policy is not None:
            properties["Policy"] = self.policy.properties(
            )

        return properties

    def ref(
        self,
    ) -> SecurityPolicyRef:
        return SecurityPolicyRef(
            self.name,
        )
    @staticmethod
    def application_access_grn(
    ) -> str:
        return "grn:gs2::system:identifier:securityPolicy:ApplicationAccess"

    def get_attr_security_policy_id(
        self,
    ) -> GetAttr:
        return GetAttr(
            self,
            "Item.SecurityPolicyId",
            None,
        )



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *
from .AppleAppStoreSetting import AppleAppStoreSetting
from .GooglePlaySetting import GooglePlaySetting
from .FakeSetting import FakeSetting
from .options.PlatformSettingOptions import PlatformSettingOptions


class PlatformSetting:
    apple_app_store: Optional[AppleAppStoreSetting] = None
    google_play: Optional[GooglePlaySetting] = None
    fake: Optional[FakeSetting] = None

    def __init__(
        self,
        options: Optional[PlatformSettingOptions] = PlatformSettingOptions(),
    ):
        self.apple_app_store = options.apple_app_store if options.apple_app_store else None
        self.google_play = options.google_play if options.google_play else None
        self.fake = options.fake if options.fake else None

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.apple_app_store is not None:
            properties["appleAppStore"] = self.apple_app_store.properties(
            )
        if self.google_play is not None:
            properties["googlePlay"] = self.google_play.properties(
            )
        if self.fake is not None:
            properties["fake"] = self.fake.properties(
            )

        return properties



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *

from ...core.model import CdkResource, Stack
from ...core.func import GetAttr

from ..ref.IdentifierRef import IdentifierRef

from .options.IdentifierOptions import IdentifierOptions


class Identifier(CdkResource):
    stack: Stack
    user_name: str

    def __init__(
        self,
        stack: Stack,
        user_name: str,
        options: Optional[IdentifierOptions] = IdentifierOptions(),
    ):
        super().__init__(
            "Identifier_Identifier_" + user_name
        )

        self.stack = stack
        self.user_name = user_name
        stack.add_resource(
            self,
        )


    def alternate_keys(
        self,
    ):
        return "userName"

    def resource_type(
        self,
    ) -> str:
        return "GS2::Identifier::Identifier"

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.user_name is not None:
            properties["UserName"] = self.user_name

        return properties

    def ref(
        self,
        client_id: str,
    ) -> IdentifierRef:
        return IdentifierRef(
            self.user_name,
            client_id,
        )

    def get_attr_client_id(
        self,
    ) -> GetAttr:
        return GetAttr(
            self,
            "Item.ClientId",
            None,
        )


    def get_attr_client_secret(
        self,
    ) -> GetAttr:
        return GetAttr(
            self,
            "ClientSecret",
            None,
        )



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from .Namespace import Namespace
from .options.NamespaceOptions import NamespaceOptions
from .GuildModel import GuildModel
from .options.GuildModelOptions import GuildModelOptions
from .Inbox import Inbox
from .options.InboxOptions import InboxOptions
from .Guild import Guild
from .options.GuildOptions import GuildOptions
from .enum.GuildJoinPolicy import GuildJoinPolicy
from .LastGuildMasterActivity import LastGuildMasterActivity
from .options.LastGuildMasterActivityOptions import LastGuildMasterActivityOptions
from .RoleModel import RoleModel
from .options.RoleModelOptions import RoleModelOptions
from .Member import Member
from .options.MemberOptions import MemberOptions
from .ReceiveMemberRequest import ReceiveMemberRequest
from .options.ReceiveMemberRequestOptions import ReceiveMemberRequestOptions
from .IgnoreUser import IgnoreUser
from .options.IgnoreUserOptions import IgnoreUserOptions
from .CurrentMasterData import CurrentMasterData


# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *
from .options.GooglePlaySettingOptions import GooglePlaySettingOptions


class GooglePlaySetting:
    package_name: Optional[str] = None
    public_key: Optional[str] = None

    def __init__(
        self,
        options: Optional[GooglePlaySettingOptions] = GooglePlaySettingOptions(),
    ):
        self.package_name = options.package_name if options.package_name else None
        self.public_key = options.public_key if options.public_key else None

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.package_name is not None:
            properties["packageName"] = self.package_name
        if self.public_key is not None:
            properties["publicKey"] = self.public_key

        return properties



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *
from .InGameLogTag import InGameLogTag
from .options.InGameLogOptions import InGameLogOptions


class InGameLog:
    timestamp: int
    request_id: str
    payload: str
    user_id: Optional[str] = None
    tags: Optional[List[InGameLogTag]] = None

    def __init__(
        self,
        timestamp: int,
        request_id: str,
        payload: str,
        options: Optional[InGameLogOptions] = InGameLogOptions(),
    ):
        self.timestamp = timestamp
        self.request_id = request_id
        self.payload = payload
        self.user_id = options.user_id if options.user_id else None
        self.tags = options.tags if options.tags else None

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.timestamp is not None:
            properties["timestamp"] = self.timestamp
        if self.request_id is not None:
            properties["requestId"] = self.request_id
        if self.user_id is not None:
            properties["userId"] = self.user_id
        if self.tags is not None:
            properties["tags"] = [
                v.properties(
                )
                for v in self.tags
            ]
        if self.payload is not None:
            properties["payload"] = self.payload

        return properties



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *
from .IgnoreUser import IgnoreUser
from .options.IgnoreUsersOptions import IgnoreUsersOptions


class IgnoreUsers:
    guild_model_name: str
    users: Optional[List[IgnoreUser]] = None
    revision: Optional[int] = None

    def __init__(
        self,
        guild_model_name: str,
        options: Optional[IgnoreUsersOptions] = IgnoreUsersOptions(),
    ):
        self.guild_model_name = guild_model_name
        self.users = options.users if options.users else None
        self.revision = options.revision if options.revision else None

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.guild_model_name is not None:
            properties["guildModelName"] = self.guild_model_name
        if self.users is not None:
            properties["users"] = [
                v.properties(
                )
                for v in self.users
            ]

        return properties



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *

from ...core.func import GetAttr, Join
from ..stamp_sheet.IncreaseCounterByUserId import IncreaseCounterByUserId
from ..stamp_sheet.SetCounterByUserId import SetCounterByUserId
from ..model.ScopedValue import ScopedValue
from ..stamp_sheet.DecreaseCounterByUserId import DecreaseCounterByUserId
from ..stamp_sheet.VerifyCounterValueByUserId import VerifyCounterValueByUserId


class CounterModelRef:
    namespace_name: str
    counter_name: str

    def __init__(
        self,
        namespace_name: str,
        counter_name: str,
    ):
        self.namespace_name = namespace_name
        self.counter_name = counter_name

    def increase_counter(
        self,
        value: int,
        user_id: Optional[str] = "#{userId}",
    ) -> IncreaseCounterByUserId:
        return IncreaseCounterByUserId(
            self.namespace_name,
            self.counter_name,
            value,
            user_id,
        )

    def set_counter(
        self,
        values: Optional[List[ScopedValue]] = None,
        user_id: Optional[str] = "#{userId}",
    ) -> SetCounterByUserId:
        return SetCounterByUserId(
            self.namespace_name,
            self.counter_name,
            values,
            user_id,
        )

    def decrease_counter(
        self,
        value: int,
        user_id: Optional[str] = "#{userId}",
    ) -> DecreaseCounterByUserId:
        return DecreaseCounterByUserId(
            self.namespace_name,
            self.counter_name,
            value,
            user_id,
        )

    def verify_counter_value(
        self,
        verify_type: str,
        scope_type: Optional[str] = None,
        reset_type: Optional[str] = None,
        condition_name: Optional[str] = None,
        value: Optional[int] = None,
        multiply_value_specifying_quantity: Optional[bool] = None,
        user_id: Optional[str] = "#{userId}",
    ) -> VerifyCounterValueByUserId:
        return VerifyCounterValueByUserId(
            self.namespace_name,
            self.counter_name,
            verify_type,
            scope_type,
            reset_type,
            condition_name,
            value,
            multiply_value_specifying_quantity,
            user_id,
        )

    def grn(
        self,
    ) -> str:
        return Join(
            ":",
            [
                "grn",
                "gs2",
                GetAttr.region(
                ).str(
                ),
                GetAttr.owner_id(
                ).str(
                ),
                "mission",
                self.namespace_name,
                "counter",
                self.counter_name,
            ],
        ).str(
        )



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from .Namespace import Namespace
from .options.NamespaceOptions import NamespaceOptions
from .TakeOverTypeModel import TakeOverTypeModel
from .options.TakeOverTypeModelOptions import TakeOverTypeModelOptions
from .OpenIdConnectSetting import OpenIdConnectSetting
from .options.OpenIdConnectSettingOptions import OpenIdConnectSettingOptions
from .PlatformUser import PlatformUser
from .options.PlatformUserOptions import PlatformUserOptions
from .BanStatus import BanStatus
from .options.BanStatusOptions import BanStatusOptions
from .CurrentMasterData import CurrentMasterData